package cs3500.connectn.model;

import cs3500.connectn.model.Model.IllegalMoveException;
import org.junit.Test;

import static cs3500.connectn.model.Player.*;
import static org.junit.Assert.*;

/**
 * Unit tests for {@link ModelImpl}.
 */
public class ModelTest {
  /**
   * An empty Connect 4 model.
   */
  Model connect4 = Model.connectFour();

  /**
   * An empty 2-by-3 Connect 3 model.
   */
  Model connect3 = Model.builder()
                    .width(2).height(3).goal(3)
                    .build();

  @Test
  public void defaultConstantsAreAsExpected() {
    assertEquals(7, Model.DEFAULT_WIDTH);
    assertEquals(6, Model.DEFAULT_HEIGHT);
    assertEquals(4, Model.DEFAULT_GOAL);
  }

  @Test
  public void widthOfConnectFourIsDefault() {
    assertEquals(Model.DEFAULT_WIDTH, connect4.getWidth());
  }

  @Test
  public void widthOfTwoByThreeIsTwo() {
    assertEquals(2, connect3.getWidth());
  }

  @Test
  public void heightOfConnectFourIsDefault() {
    assertEquals(Model.DEFAULT_HEIGHT, connect4.getHeight());
  }

  @Test
  public void heightOfTwoByThreeIsThree() {
    assertEquals(3, connect3.getHeight());
  }

  @Test
  public void goalOfConnectFourIsDefault() {
    assertEquals(Model.DEFAULT_GOAL, connect4.getGoal());
  }

  @Test
  public void goalOfCorrectThreeIsThree() {
    assertEquals(3, connect3.getGoal());
  }

  @Test
  public void newGameIsNotOver4() {
    assertFalse(connect4.isGameOver());
  }

  @Test
  public void newGameIsNotOver3() {
    assertFalse(connect3.isGameOver());
  }

  @Test
  public void firstPlayerIsWhite4() {
    assertEquals(White, connect4.getNextPlayer());
  }

  @Test
  public void firstPlayerIsWhite3() {
    assertEquals(White, connect3.getNextPlayer());
  }

  @Test(expected = IllegalStateException.class)
  public void getWinnerThrowsWhenGameNotOver4() {
    connect4.getWinner();
  }

  @Test(expected = IllegalStateException.class)
  public void getWinnerThrowsWhenGameNotOver3() {
    connect3.getWinner();
  }

  @Test
  public void emptyHasNoPlayers4() {
    assertNull(connect4.getPlayerAt(0, 0));
    assertNull(connect4.getPlayerAt(connect4.getWidth() - 1,
                                    connect4.getHeight() - 1));
  }

  @Test
  public void emptyHasNoPlayers3() {
    assertNull(connect3.getPlayerAt(0, 0));
    assertNull(connect3.getPlayerAt(connect3.getWidth() - 1,
                                    connect3.getHeight() - 1));
  }

  @Test(expected = IndexOutOfBoundsException.class)
  public void getPlayerThrowsIfWidthOob4() {
    assertNull(connect4.getPlayerAt(connect4.getWidth(),
                                    connect4.getHeight() - 1));
  }

  @Test(expected = IndexOutOfBoundsException.class)
  public void getPlayerThrowsIfWidthOob3() {
    assertNull(connect3.getPlayerAt(connect3.getWidth(),
                                    connect3.getHeight() - 1));
  }

  @Test(expected = IndexOutOfBoundsException.class)
  public void getPlayerThrowsIfHeightOob4() {
    assertNull(connect4.getPlayerAt(connect4.getWidth() - 1,
                                    connect4.getHeight()));
  }

  @Test(expected = IndexOutOfBoundsException.class)
  public void getPlayerThrowsIfHeightOob3() {
    assertNull(connect3.getPlayerAt(connect3.getWidth() - 1,
                                    connect3.getHeight()));
  }

  @Test
  public void moveIntoOccupiedThrows() throws IllegalMoveException {
    assertEquals(0, connect4.move(White, 2));
    assertEquals(White, connect4.getPlayerAt(2, 0));
    assertNull(connect4.getPlayerAt(2, 1));
  }

  @Test(expected = IllegalStateException.class)
  public void wrongPlayerMoveThrows() throws IllegalMoveException {
    connect4.move(Red, 2);
  }

  @Test(expected = IllegalStateException.class)
  public void playAdvancesAfterMove() throws IllegalMoveException {
    connect4.move(White, 2);
    connect4.move(White, 2);
  }

  @Test
  public void horizontalLineWins() throws IllegalMoveException {
    assertEquals(0, connect4.move(White, 2));
    assertEquals(1, connect4.move(Red, 2));
    assertEquals(0, connect4.move(White, 1));
    assertEquals(1, connect4.move(Red, 1));
    assertEquals(0, connect4.move(White, 4));
    assertEquals(1, connect4.move(Red, 4));
    assertFalse(connect4.isGameOver());
    assertEquals(0, connect4.move(White, 3));
    assertTrue(connect4.isGameOver());
    assertEquals(White, connect4.getWinner());
  }

  @Test
  public void verticalLineWins() throws IllegalMoveException {
    assertEquals(0, connect4.move(White, 2));
    assertEquals(0, connect4.move(Red, 3));
    assertEquals(1, connect4.move(White, 2));
    assertEquals(1, connect4.move(Red, 3));
    assertEquals(2, connect4.move(White, 2));
    assertEquals(2, connect4.move(Red, 3));
    assertEquals(0, connect4.move(White, 0));
    assertFalse(connect4.isGameOver());
    assertEquals(3, connect4.move(Red, 3));
    assertTrue(connect4.isGameOver());
    assertEquals(Red, connect4.getWinner());
  }

  @Test
  public void diagonalWins() throws IllegalMoveException {
    assertEquals(0, connect4.move(White, 0));
    assertEquals(0, connect4.move(Red, 1));
    assertEquals(0, connect4.move(White, 2));
    assertEquals(1, connect4.move(Red, 2));
    assertEquals(2, connect4.move(White, 2));
    assertEquals(0, connect4.move(Red, 3));
    assertEquals(1, connect4.move(White, 3));
    assertEquals(2, connect4.move(Red, 3));
    assertEquals(3, connect4.move(White, 3));
    assertEquals(4, connect4.move(Red, 3));
    assertFalse(connect4.isGameOver());
    assertEquals(1, connect4.move(White, 1));
    assertTrue(connect4.isGameOver());
    assertEquals(White, connect4.getWinner());
  }

  @Test
  public void canFillColumn() throws IllegalMoveException {
    assertEquals(0, connect4.move(connect4.getNextPlayer(), 0));
    assertEquals(1, connect4.move(connect4.getNextPlayer(), 0));
    assertEquals(2, connect4.move(connect4.getNextPlayer(), 0));
    assertEquals(3, connect4.move(connect4.getNextPlayer(), 0));
    assertEquals(4, connect4.move(connect4.getNextPlayer(), 0));
    assertEquals(5, connect4.move(connect4.getNextPlayer(), 0));
  }

  @Test(expected = IllegalMoveException.class)
  public void cannotOverfillColumn() throws IllegalMoveException {
    canFillColumn();
    connect4.move(connect4.getNextPlayer(), 0);
  }

  private void movesAlternatePlayers() throws IllegalMoveException {
    connect3.move(White, 0);
    connect3.move(Red, 1);
    connect3.move(White, 0);
    connect3.move(Red, 1);
  }

  @Test
  public void gameCanEndInStalemate() throws IllegalMoveException {
    canFillColumn();
    connect3.move(White, 1);
    assertFalse(connect3.isGameOver());
    connect3.move(Red, 0);
    assertTrue(connect3.isGameOver());
    assertEquals(ModelImpl.Status.Stalemate, connect3.getStatus());
  }

  @Test(expected = IllegalStateException.class)
  public void cannotMovePastEndOfGame() throws IllegalMoveException {
    gameCanEndInStalemate();
    connect3.move(White, 0);
  }

  @Test(expected = IllegalStateException.class)
  public void overfillColumnThrows() throws IllegalMoveException {
    canFillColumn();
    connect3.move(White, 0);
    connect3.move(Red, 0);
  }
}